from hypothesis_jsonschema import from_schema
from thd_json.projection import get_projection_validator
from thd_json.header import get_header_validator, generate_header
from thd_json.source import get_source_validator
from thd_json.detector import get_detector_validator
from thd_json.roi import get_roi_validator
from thd_json.joint import get_joint_validator
from thd_json.pixel_coords import get_pixel_coord_validator
from thd_json.spatial_points import get_spatial_point_validator
from thd_json.reprojection_calibration import get_reprojection_calibration_validator
from hypothesis import given, settings
import json
from pathlib import Path


SAVE_FOLDER = Path("./examples/generated")


def set_uuid(src: dict):
    for k, v in src.items():
        if isinstance(v, dict):
            set_uuid(v)

        if k == "uuid":
            header = generate_header()
            src[k] = header.uuid


def main():
    for name, get_validator in [
        ("projection", get_projection_validator),
        ("header", get_header_validator),
        ("source", get_source_validator),
        ("detector", get_detector_validator),
        ("roi", get_roi_validator),
        ("joint", get_joint_validator),
        ("pixel", get_pixel_coord_validator),
        ("point", get_spatial_point_validator),
        ("calib", get_reprojection_calibration_validator),
    ]:
        validator = get_validator()
        schema = validator.get_schema()
        strategy = from_schema(schema)

        @settings(max_examples=1)
        @given(strategy)
        def schema(data):
            with open(SAVE_FOLDER / f"example_{name}.json", "w") as f:
                set_uuid(data)
                json.dump(data, f, indent=4)
            return

        schema()


if __name__ == "__main__":
    main()
